function mdl_color(varargin)
%MDL_COLOR Remove or return color to Simulink object backgrounds
%  MDL_COLOR('off') will set the background color of all objects in the 
%  current model to white. It will record the colors of the blocks before
%  they were converted and store that information in a base workspace 
%  variable "<bdroot>_mdl_color_record."
%
%  MDL_COLOR('on') will take a previously created <bdroot>_mdl_color_record
%  variable in the base workspace and use it to set object background 
%  colors to those specified in the variable. In this manner, the color of 
%  objects previously whited out will return to what they were. This 
%  command may also be used to automate coloring of blocks if the user 
%  creates a <bdroot>_mdl_color_record variable by hand.
%
%  Calling MDL_COLOR with no input argument will either remove or return 
%  color to the current Simulink model depending on which action is 
%  appropriate at the time.
%
%  NOTE: It is important that you keep the variable 
%  <bdroot>_mdl_color_record in the base workspace. Deleting or renaming 
%  the variable will prevent MDL_COLOR from returning colors to your model.
%  Be careful to avoid clear commands or shutting down MATLAB. If in doubt 
%  about the security of mdl_color_record, it is advised that you save it 
%  to a mat file.
%
%  Examples: 
%    mdl_color off, pause, mdl_color on
%    sets all background colors to white in the current model before 
%    returning the colors to their original settings
%
% Coded by Will Campbell, MathWorks Inc.
% will.campbell@mathworks.com
% January 22, 2010
% Copyright 2010 The MathWorks, Inc.

if nargin == 0
    if evalin('base',['exist(''' bdroot '_mdl_color_record'',''var'')'])
        % There's a valid variable in the workspace
        % The user probably wants to revive color
        flag = 'on';
    else
        flag = 'off';
    end
elseif nargin > 1
    error('Too many inputs to mdl_color')
else
    flag = varargin{1};
end

if strcmpi(flag,'off')
    % Find all objects that have a parameter "BackgroundColor"
    A = find_system(bdroot,'RegExp','on','LookUnderMasks','all','BackgroundColor','.');
    % Find all objects that have a parameter "BackgroundColor" that is set to white
    B = find_system(bdroot,'LookUnderMasks','all','BackgroundColor','white'); % won't work with [1 1 1]
    % Find the difference between A and B
    C = setdiff(A,B);
    % Record the colors of non-white blocks
    colors = get_param(C,'BackgroundColor');
    % Send to the workspace
    assignin('base',[bdroot '_mdl_color_record'],[C colors]);
    % Set all non-white blocks to be white
    set_params(C,'BackgroundColor','white')
elseif strcmpi(flag,'on')
    % See if mdl_color_record is in the base
    try
        mdl_color_record = evalin('base',[bdroot '_mdl_color_record']);
    catch
        error('Cannot revive color because there is no mdl_color_record variable in the base workspace.')
    end
    
    % Revive colors
    for i = 1:size(mdl_color_record,1)
        set_param(mdl_color_record{i,1},'BackgroundColor',mdl_color_record{i,2})
    end
    
    % Remove variable from base workspace
    evalin('base',['clear ' bdroot '_mdl_color_record'])
else
    error('Invalid input to mdl_color. Value must be "on" or "off."')
end
